/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : chromo.h                                                 */
/*                                                                       */
/*       Date : November  6, 1997                                        */
/* Updated at : November 24, 1997                                        */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/

#ifndef _CHROMO_
#define _CHROMO_

#include "global.h"
#include "gene.h"

struct struct_Chromosome
{

   unsigned int chromoLength;     /* chromosome length */
   eval_t myEval;                 /* chromosome evaluation */
   eval_t myModEval;              /* chromosome modified evaluation (fitness) */
   type_Gene **genePool;          /* pointer to original gene pool vector */
   type_Gene *myGenePool;         /* private gene pool vector */

};

typedef struct struct_Chromosome type_Chromosome;

#include "pop.h"

type_Chromosome* copyChromosome(type_Chromosome* RHS); 

type_Chromosome* defineChromosome(unsigned int _chromoLength, 
                                  type_Gene *_genePool[]);

void init(type_Chromosome* this, type_Population* myPop);

/*
// Calculates (if necessary) and returns chromosome evaluation.
*/
eval_t eval(type_Chromosome* this);

/*
// Sets chromosome's modified evaluation (fitness).
*/
eval_t setModEval(type_Chromosome* this, eval_t _modEval); 

/*
// Returns chromosome's modified evaluation (fitness).
*/
eval_t getModEval(type_Chromosome* this); 

/*
// Returns chromosome's evaluation (fitness).
*/
eval_t getEval(type_Chromosome* this); 


/*
// Mates this chromosome with partner resulting in two new 
// offsprings, child1 and child2.
*/
void mate(type_Chromosome* this, type_Chromosome* partner, float crossover, 
          enum cross_t crossoverType, float mutation, 
          type_Chromosome *child1, type_Chromosome *child2);

/*  Returns chromosome length.  */
unsigned int getLength(type_Chromosome* this); 

int printChromosome(type_Chromosome* this);

void disposeChromosome(type_Chromosome* this);

#endif

